@@ -112,9 +112,12 @@ int main(int argc, char **argv) {
 		}
 	}
 
-	if (query == NULL) {
-		query = (char *)malloc(14+strlen(table));
-		sprintf(query, "SELECT * FROM %s", table);
+	argc -= optind;
+	argv = &argv[optind];
+
+	if (argc != 1) {
+		usage();
+		exit(1);
 	}
 
 	if ((dbase == NULL) || ((table == NULL) && (query == NULL))) {
@@ -123,12 +126,9 @@ int main(int argc, char **argv) {
 		exit(1);
 	}
 
-	argc -= optind;
-	argv = &argv[optind];
-
-	if (argc != 1) {
-		usage();
-		exit(1);
+	if (query == NULL) {
+		query = (char *)malloc(14+strlen(table));
+		sprintf(query, "SELECT * FROM %s", table);
 	}
 
 	if (verbose > 1) {
@@ -191,13 +191,15 @@ int main(int argc, char **argv) {
 	while ((qfield = msqlFetchField(qres))) {
 		strtoupper(qfield->name);
 		switch (qfield->type) {
+/* assume INT's have a max. of 10 digits (4^32 has 10 digits) */
 			case INT_TYPE:
-				dbf_add_field(dbh, qfield->name, 'N', qfield->length, 0);
+				dbf_add_field(dbh, qfield->name, 'N', 10, 0);
 				if (verbose > 1) {
 					printf("Adding field: %s, INT_TYPE, %d\n", qfield->name,
 															   qfield->length);
 				}
 				break;
+/* do we have to make the same assumption here? */
 			case REAL_TYPE:
 				dbf_add_field(dbh, qfield->name, 'N', qfield->length,
 								(int) precision);
@@ -229,7 +231,11 @@ int main(int argc, char **argv) {
 		while ((qrow = msqlFetchRow(qres)) != NULL) {
 			if ((rec = dbf_build_record(dbh)) != (field *)DBF_ERROR) {
 				for (t = 0; t < numfields; t++) {
-					rec[t].db_contents = strdup(qrow[t]);
+					if (qrow[t] != 0) {
+						strcpy(rec[t].db_contents, qrow[t]);
+					} else {
+						rec[t].db_contents[0] = '\0';
+					}
 					if (upper) {
 						strtoupper(rec[t].db_contents);
 					}
